@@ -37,7 +37,7 @@ from awkward.contents.content import (
     RemoveStructureOptions,
     ToArrowOptions,
 )
-from awkward.forms.form import Form
+from awkward.forms.form import Form, FormKeyPathT
 from awkward.forms.regularform import RegularForm
 from awkward.index import Index
 
@@ -211,6 +211,14 @@ class RegularArray(RegularMeta[Content], Content):
             form_key=form_key,
         )
 
+    def _form_with_key_path(self, path: FormKeyPathT) -> RegularForm:
+        return self.form_cls(
+            self._content._form_with_key_path((*path, None)),
+            self._size,
+            parameters=self._parameters,
+            form_key=repr(path),
+        )
+
     def _to_buffers(
         self,
         form: Form,
@@ -358,7 +366,6 @@ class RegularArray(RegularMeta[Content], Content):
             nextcarry = ak.index.Index64.empty(
                 where.shape[0] * self._size, self._backend.index_nplike
             )
-
         assert nextcarry.nplike is self._backend.index_nplike
         self._maybe_index_error(
             self._backend[
@@ -472,6 +479,7 @@ class RegularArray(RegularMeta[Content], Content):
             nexthead, nexttail = ak._slicing.head_tail(tail)
             nextcarry = ak.index.Index64.empty(self._length, index_nplike)
             assert nextcarry.nplike is index_nplike
+            head = ak._slicing.normalize_integer_like(head)
             self._maybe_index_error(
                 self._backend[
                     "awkward_RegularArray_getitem_next_at", nextcarry.dtype.type
@@ -647,7 +655,6 @@ class RegularArray(RegularMeta[Content], Content):
                         advanced.data,
                         regular_flathead.data,
                         self._length,
-                        regular_flathead.length,
                         self._size,
                     ),
                     slicer=head,
@@ -996,7 +1003,7 @@ class RegularArray(RegularMeta[Content], Content):
             )
             self._backend.maybe_kernel_error(
                 self._backend[
-                    "awkward_RegularArray_reduce_nonlocal_preparenext",
+                    "awkward_RegularArray_reduce_nonlocal_preparenext_64",
                     nextcarry.dtype.type,
                     nextparents.dtype.type,
                     parents.dtype.type,
@@ -1067,7 +1074,7 @@ class RegularArray(RegularMeta[Content], Content):
             assert nextparents.nplike is index_nplike
             self._backend.maybe_kernel_error(
                 self._backend[
-                    "awkward_RegularArray_reduce_local_nextparents",
+                    "awkward_RegularArray_reduce_local_nextparents_64",
                     nextparents.dtype.type,
                 ](
                     nextparents.data,
@@ -1336,7 +1343,7 @@ class RegularArray(RegularMeta[Content], Content):
             )
 
             content_type = pyarrow.list_(paarray.type).value_field.with_nullable(
-                akcontent.is_option
+                akcontent._arrow_needs_option_type()
             )
 
             return pyarrow.Array.from_buffers(
